/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 * Klasse beinhaltet die Parameter welche Anzeigen ihren Inhalt in Abhngigkeit vom ermittelten Inhalt anderer
 * Anzeigen haben (Beispiel: Wenn an Anzeige X der Inhalt A angefordert wird, muss an Anzeige Y der Inhalt B
 * angefordert werden)
 * @version $Revision: 1.3 $ / $Date: 2013/06/27 13:36:29 $ / ($Author: ObertM $)
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class VerwaltungParaAbhaengigkeiten
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaAbhaengigkeiten _instance;

  /**
   * Liste mit den Anzeigen die ermittelt werden sollen
   */
  private List<Abhaengigkeiten> m_abhaenigkeiten = new ArrayList<Abhaengigkeiten>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);
  
  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaAbhngigkeiten";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaAbhaengigkeiten ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaAbhaengigkeiten getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaAbhaengigkeiten.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaAbhaengigkeiten ();
        }
      }
    }
    
    return _instance;
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  /**
   * Methode liefert alle Abhngigkeiten
   * @return Liste mit den Abhngigkeiten
   */
  public List<Abhaengigkeiten> getAbhaengigkeiten ()
  {
    List<Abhaengigkeiten> l = new ArrayList<Abhaengigkeiten>();
    
    synchronized (m_abhaenigkeiten)
    {
      for (Abhaengigkeiten a : m_abhaenigkeiten)
        l.add( a );
    }
    
    return l;  
  }

  /**
   * Methode liefert eine Liste mit allen abhngigen Anzeigen, bei denen die bergebene Anzeige die 
   * Hauptanzeige ist
   * @param anzeige Anzeige die geprft werden soll
   * @return Liste mit abhngigen Anzeigen
   */
  public Vector<SystemObject> getAbhaenigeAnzeigen (SystemObject anzeige)
  {
    Vector<SystemObject> l = new Vector<SystemObject>();
    
    synchronized (m_abhaenigkeiten)
    {
      for (Abhaengigkeiten a : m_abhaenigkeiten)
      {
        if (a.getHauptAnzeige().equals( anzeige ))
          l.add( a.getAbhaengigeAnzeige() );
      }
    }
    
    return l;
  }
  
  /**
   * Methode liefert eine Liste mit allen Hauptanzeigen bei denen die bergebene Anzeige die 
   * abhngige Anzeige ist
   * @param anzeige Anzeige die geprft werden soll
   * @return Liste mit Hauptanzeigen
   */
  public Vector<SystemObject> getHauptAnzeigen (SystemObject anzeige)
  {
    Vector<SystemObject> l = new Vector<SystemObject>();
    
    synchronized (m_abhaenigkeiten)
    {
      for (Abhaengigkeiten a : m_abhaenigkeiten)
      {
        if (a.getAbhaengigeAnzeige().equals( anzeige ))
          l.add( a.getHauptAnzeige() );
      }
    }
    
    return l;
  }
  
  
  /**
   * Methode prft ob es sich bei der Anzeige um eine "Abhngige Anzeige" handelt
   * @param anzeige zu prfende Anzeige
   * @return Anzeige ist eine abhngige Anzeige
   */
  public boolean isAbhaengigeAnzeige (SystemObject anzeige)
  {
    return getHauptAnzeigen( anzeige ).size() > 0;
  }

  /**
   * Methode prft ob es sich bei der Anzeige um ein "Hauptanzeige" handelt
   * @param anzeige zu prfende Anzeige
   * @return Anzeige ist eine Hauptanzeige
   */
  public boolean isHauptAnzeige (SystemObject anzeige)
  {
    return getAbhaenigeAnzeigen( anzeige ).size() > 0;
  }

  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaAbhngigkeiten'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_abhaenigkeiten)
    {
      m_abhaenigkeiten.clear();
      
      if (dat.hasData())
      {
        Array a = dat.getData().getArray( "Abhngigkeiten" );
    
        int anzahl = a.getLength();
        for (int x = 0; x < anzahl; ++x)
        {
          Abhaengigkeiten ah = new Abhaengigkeiten();
          ah.auswerteAttributGruppe (a.getItem( x ));
          
          m_abhaenigkeiten.add( ah );
        }
      }
    }
  }

  /**
   * Methode fgt eine Abhngigkeit dem Parametersatz hinzu
   * @param hauptAnzeige Hauptanzeige
   * @param abhaengigeAnzeige von der Hauptanzeige abhngige Anzeige
   */
  public void addAbhaengigkeit (SystemObject hauptAnzeige, SystemObject abhaengigeAnzeige)
  {
    Abhaengigkeiten abhaengigkeit = new Abhaengigkeiten ();
    
    abhaengigkeit.setHauptAnzeige( hauptAnzeige );
    abhaengigkeit.setAbhaengigeAnzeige( abhaengigeAnzeige );
    
    synchronized (m_abhaenigkeiten)
    {
      m_abhaenigkeiten.add( abhaengigkeit );
    } 
  }

  /**
   * Methode bernimmt die Parameter
   * @param alleRegeln zu bernehmende Parameter
   */
	public void setParameter(Vector<ListeMitRegeln> alleRegeln)
  {
		synchronized (m_abhaenigkeiten)
    {
			m_abhaenigkeiten.clear();
			
			for (ListeMitRegeln lmr : alleRegeln)
			{
				Vector<Abhaengigkeiten> abhaengigkeiten = lmr.getParameter();
				
				for (Abhaengigkeiten a : abhaengigkeiten)
			    m_abhaenigkeiten.add(a);		
			}
    }
  }
	
  /**
   * Methode zum Speichern der Parameter "Abhngigkeiten" im DaV
   * @param abhaengigkeiten Abhngigkeiten
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(_pidAtg);
    Data data = connection.createData(atg );
    
    synchronized (m_abhaenigkeiten)
    {
	    Array a = data.getArray( "Abhngigkeiten" );
	    a.setLength( m_abhaenigkeiten.size() );
	    
	    for (int i=0; i<m_abhaenigkeiten.size(); ++i)
	    {
	      Data aktuell = a.getItem( i );
	      
	      aktuell.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( null );
	      aktuell.getReferenceValue( "HauptAnzeige" ).setSystemObject( m_abhaenigkeiten.get( i ).getHauptAnzeige() );
	      aktuell.getReferenceValue( "AbhngigeAnzeige" ).setSystemObject( m_abhaenigkeiten.get( i ).getAbhaengigeAnzeige() );
	      
	      WVZInhalt dummy = VerwaltungWvzInhalte.getInstance().getWvzInhaltDontCare();
	      
	      dummy.erzeugeAttributgruppe( aktuell.getItem( "ZeichenfolgeHauptAnzeige" ) );
	      dummy.erzeugeAttributgruppe( aktuell.getItem( "ZeichenfolgeAbhngigeAnzeige" ) );
	    }
    }

    return data;
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }

  /**
   * Methode liefert die Parameter als Liste mit regeln
   * @return Parameter als Liste mit Regeln
   */
	public ListeMitRegeln getParameter()
  {
		ListeMitRegeln lmr = new ListeMitRegeln();
		
		synchronized (m_abhaenigkeiten)
    {
	    Vector<Abhaengigkeiten> v = new Vector<Abhaengigkeiten>();
	    for (Abhaengigkeiten a : m_abhaenigkeiten)
	    	v.add(a);
	   
	    lmr.setParameter(v);
    }
	  return lmr;
  }


}
