/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import de.sts.buv.plugin.workflow.tsf.dav.WiedervorlageTSFWorkflowAbschnitt;

/**
 * Klasse zur Registrierung von {@link IWiedervorlage}n. Wiedervorgelegt werden knnen z.Zt. Schaltempfehlungen des
 * Workflows sowie Aufforderungen zum Weiterschalten.
 * 
 * @author HofmannJ
 * 
 */
public class RegistryWiedervorlagen
{
  private static RegistryWiedervorlagen _instance = null;

  /**
   * Key = ID des wiedervorzulegenden Elementes (i.d.R. die Pid), Value = Instanz von {@link IWiedervorlage}. Der
   * Zugriff auf diese Map sollte synchronisiert erfolgen.
   */
  private TreeMap<String, IWiedervorlage> _map = new TreeMap<String, IWiedervorlage>();

  /**
   * Listener-Untersttzung.
   */
  private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

  /**
   * Wird an einen neu angemeldeten Listener mit einem Array von {@link IWiedervorlage}-Objekten als New-Value
   * bermittelt.
   */
  public static final String PROP_INIT = "INIT";

  /**
   * Wird an alle angemeldeten Listener bermittelt mit einem {@link IWiedervorlage}-Objekt als New-Value. Im Old-Value
   * wird das vorhergehende Element unter dessen ID bermittelt sofern verfgbar, sonst <code>null</code>.
   */
  public static final String PROP_ADD = "ADD";

  /**
   * Wird an alle angemeldeten Listener bermittelt, wenn ein {@link IWiedervorlage} entfernt wurde. Im Old-Value ist
   * das entfernte {@link IWiedervorlage}-Element hinterlegt, New-Value ist <code>null</code>.
   */
  public static final String PROP_REMOVE = "REMOVE";

  /**
   * Privater Konstruktor.
   */
  private RegistryWiedervorlagen()
  {
  }

  /**
   * Liefert die Singleton-Instanz dieser Klasse.
   * 
   * @return Singleton-Instanz dieser Klasse.
   */
  public static RegistryWiedervorlagen getInstance()
  {
    if (_instance == null)
    {
      synchronized (RegistryWiedervorlagen.class)
      {
        if (_instance == null)
        {
          _instance = new RegistryWiedervorlagen();
        }
      }
    }

    return _instance;
  }

  public void addWiedervorlage(IWiedervorlage wiedervorlage)
  {
    if ((wiedervorlage != null) && (wiedervorlage.getId() != null))
    {
      IWiedervorlage oldValue = null;

      synchronized (_map)
      {
        if (_map.containsKey(wiedervorlage.getId()))
        {
          oldValue = _map.get(wiedervorlage.getId());
        }
        _map.put(wiedervorlage.getId(), wiedervorlage);
      }

      if ((oldValue != null) && (oldValue instanceof WiedervorlageTSFWorkflowAbschnitt))
      {
        ((WiedervorlageTSFWorkflowAbschnitt)oldValue).dispose();
      }

      _pcs.firePropertyChange(PROP_ADD, oldValue, wiedervorlage);
    }
  }

  public void removeWiedervorlage(IWiedervorlage wiedervorlage)
  {
    if ((wiedervorlage != null) && (wiedervorlage.getId() != null))
    {
      IWiedervorlage removed;
      synchronized (_map)
      {
        removed = _map.remove(wiedervorlage.getId());
      }

      if (removed != null)
      {
        _pcs.firePropertyChange(PROP_REMOVE, removed, null);
      }
    }
  }

  public IWiedervorlage[] getWiedervorlagen()
  {
    LinkedList<IWiedervorlage> liste = new LinkedList<IWiedervorlage>();

    synchronized (_map)
    {
      Iterator<Entry<String, IWiedervorlage>> it = _map.entrySet().iterator();
      while (it.hasNext())
      {
        Entry<String, IWiedervorlage> entry = it.next();
        liste.add(entry.getValue());
      }
    }

    return liste.toArray(new IWiedervorlage[liste.size()]);
  }

  public void addListener(PropertyChangeListener pcl)
  {
    if (pcl != null)
    {
      _pcs.addPropertyChangeListener(pcl);

      IWiedervorlage[] aWiedervorlagen = getWiedervorlagen();
      if (aWiedervorlagen.length > 0)
      {
        pcl.propertyChange(new PropertyChangeEvent(this, PROP_INIT, null, aWiedervorlagen));
      }
    }
  }

  public void removeListener(PropertyChangeListener pcl)
  {
    if (pcl != null)
    {
      _pcs.removePropertyChangeListener(pcl);
    }
  }

  @Override
  public String toString()
  {
    IWiedervorlage[] aWiedervorlagen = getWiedervorlagen();

    StringBuffer sb = new StringBuffer();

    sb.append("Anzahl registrierter Wiedervorlagen: ");
    sb.append(aWiedervorlagen.length);
    sb.append("\n");

    if (aWiedervorlagen.length > 0)
    {
      for (int i = 0; i < aWiedervorlagen.length; i++)
      {
        sb.append("\tWiedervorlage ID=");
        sb.append(aWiedervorlagen[i].getId());
        sb.append(", Beschreibung=");
        sb.append(aWiedervorlagen[i].getBeschreibung());

        if ((i + 1) < aWiedervorlagen.length)
        {
          sb.append("\n");
        }
      }
    }

    return sb.toString();
  }

}
